/*
** Module   : ScrollableSelectionListGuiImpl.java
** Abstract : scrollable selection list implementation. 
**
** Copyright (c) 2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description---------------------------------
** 001 EVL 20150423 Initial version.
*/

package com.goldencode.p2j.ui.client.gui;

import java.awt.Color;
import java.awt.event.*;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.chui.ThinClient;
import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.ScrollBar.Position;
import com.goldencode.p2j.ui.client.event.*;
import com.goldencode.p2j.ui.client.gui.GuiFontResolver.FontCache;
import com.goldencode.p2j.ui.client.gui.driver.*;
import com.goldencode.p2j.ui.client.model.*;
import com.goldencode.p2j.ui.client.widget.*;
import com.goldencode.p2j.util.*;

/**
 * Scrollable selection list GUI implementation.  To be used to support combo box and selection
 * list widgets. 
 */
@SuppressWarnings("unchecked")
public class ScrollableSelectionListGuiImpl
extends ScrollableList<GuiOutputManager>
{
   /** Screen driver */
   private GuiDriver gd;

   /** Widget configuration */
   private BaseConfig config;

   /** Color resolver */
   private GuiColorResolver gc;
   
   /** Font resolver */
   private GuiFontResolver gf;
   
   /** Row height */
   private int rowHeight;

   /** The vertical scroll bar associated with this scrollable list. */
   private ScrollBar<GuiOutputManager> vbar;

   /** Selected item box color. */
   private Color selectionBack;

   /** Selected item text color. */
   private Color selectionFore;

   /** Selected dotted line color. */
   private Color selectionDots;

   /**
    * Create component using given configuration and data model.
    * 
    * @param    box
    *           The instance with which this object must be associated.
    * @param    model
    *           The data model that represents the contents of the list.
    */
   ScrollableSelectionListGuiImpl(ComboBox<GuiOutputManager> box, ListModel<String> model)
   {
      super(box, model);
      // initialize screen driver
      gd = (GuiDriver) OutputManager.getDriver();
      // and font resolver
      gf = new GuiFontResolver(Window.resolveWindow(this), gd, box.config(), false);
      // color initializing
      selectionBack = gd.getSysColor(SysColor.COLOR_HIGHLIGHT);
      selectionFore = gd.getSysColor(SysColor.COLOR_HIGHLIGHTTEXT);
   }

   /**
    * Request focus.
    */
   @Override
   public void requestFocus()
   {
      super.requestFocus();
//      screen().at(screenLocation().translate(0, currentRow()));
   }

   /**
    * Get widget width.
    * 
    * @return  widget width.
    */
/*   @Override
   public double width()
   {
      boolean parentWidth = (isInheritWidth() && parent() != null);
      return (parentWidth) ? parent().width() : super.width();
   }*/

   /**
    * Draws this component. Overrides draw() in JList.
    */
   @Override
   public void draw()
   {
      // combo-box options need to be refreshed before every painting
      final ComboBoxConfig cbbcfg = box.config();
      gc = new GuiColorResolver(gd, cbbcfg, SysColor.COLOR_MENU, SysColor.COLOR_BTNTEXT);

      // determine colors to use
      gc.colors();
      
      // resolve font
      final FontCache fntCache = gf.font();

      // get location and size
      Point origin = screenLocation();
      NativePoint p = physicalLocation();
//      NativePoint p = screen().coordinates().pointToNative(origin);
      final NativeDimension d = physicalDimension();
      final int rowHeight = screen().coordinates().heightToNative(1.0); 
//      d.height = visibleRows() * (fntCache.font.getSize() + 2);
      d.height = visibleRows() * (rowHeight);

      // set up clipping rectangle
      NativeRectangle clip = clipRectangle(p, d);

      // clippable drawing start 
      gd.draw(p, clip, new Runnable()
      {

         @Override
         public void run()
         {
            // inside this call widget origin is always (0,0)
            NativePoint po = new NativePoint(0, 0);

//            gd.setColor(gc.fgColor);
//            gd.drawRect(po.x, po.y, d.width, d.height);
//            gd.setColor(gc.bgColor);
//            gd.fillRect(po.x + 1, po.y + 1, d.width - 2, d.height - 2);
            // draw entry field text
            gd.setGuiFont(fntCache.font);
            gd.setFontStyle(fntCache.font.getStyle());
//            gd.setColor(gc.fgColor);
            
            int x = po.x;
            int y = po.y;
            for (int i = 0; i < model().size() && i < visibleRows();
//                            i++, y+= fntCache.font.getSize())
                            i++, y+= rowHeight)
            {
               if (y < 0)
                  continue;
         
//               if (selected(i))
               if (i == currentRow())
               {
                  gd.setColor(selectionBack);
                  gd.fillRect(x, y, d.width, rowHeight);
                  gd.setColor(selectionFore);
               }
               else
               {
                  gd.setColor(gc.fgColor);
               }
               
               gd.drawString(getItemText(i), x, y + (rowHeight + fntCache.font.getSize())/2);
            }
            // button state flags
         }
         
      });
/*      Point origin = screenLocation();
      Color dc  = box.getDColor();
      
      String blanks = StringHelper.repeatChar(' ', (int) width());
      
      // in chui char coordinates are always whole numbers
      int y = (int) location().y;
      
      for (int i = 0; i < model().size() &&
                      y < visibleRows(); i++, y++, origin.y++)
      {
         if (y < 0)
            continue;
         
         screen().at(origin);

         Color color = (i == currentRow()) ? dc.withReverse() : dc;

         StringBuilder buf = new StringBuilder(getItemText(i));
         
         buf.append(blanks).setLength((int) width());

         screen().append(buf.toString(), color);
      }

      for (; y < visibleRows(); y++, origin.y++)
      {
         Color color = (y == 0 && currentRow() == -1) ? dc.withReverse() : dc;

         screen().at(origin);
         screen().append(blanks, color);
      }
*/
//      if (hasFocus() && isEnabled())
//         drawCaret();
   }

   /**
    * Process scroll event.
    * 
    * @param   event
    *          Scroll event.
    */
   @Override
   public void processScrollEvent(ScrollEvent event)
   {
      super.ensureRowVisible((int) event.limit().y, false);       
      repaint();
   }

   /**
    * Get the widget config.
    * 
    * @return   See above.
    */
//   @Override
//   public BaseConfig config()
//   {
//      return box.config();
//   }

   /**
    * Set the list's vertical scrollarr.
    * 
    * @param    vbar
    *           The vertical scrollbar.
    */
   public void setScrollBar(ScrollBar<GuiOutputManager> vbar)
   {
      this.vbar = vbar;
   }

   /**
    * Sets position of terminal's cursor to the first char of the combo-box.
    */
   @Override
   public void drawCaret()
   {
      // this check is required since the method can be called from multiple
      // drivers
      if (!hasFocus() || !isEnabled())
      {
         return;
      }

      Point origin = screenLocation();

      if (currentRow() > 0)
      {
          origin.y += currentRow();
      }

//      screen().at(origin);
//      requestSync();
   }
}
